#include <gtk/gtk.h>
//#include <math.h>
#include <string.h>
#include "../Misc/my_dbug.h"
#include "../Misc/mem_management.h"
#include "../Misc/garbage_collection.h"
#include "../Syntaxtree/syntaxtree.h"
#include "pattern_matcher.h"
#include "eval_global.h"
#include "eval_expand.h"
#include "evaluator.h"
#include "../Syntaxtree/syntaxtree.h"



void
replaceAllOccurencesAndSetExpansiondepth(gchar* ofVar, 
					 Expr_node_ptr parent_of_inExpr, 
					 Expr_node_ptr inExpr, 
					 Expr_node_ptr byExpr, 
					 gint expansiondepth)
{
  gint i;
  Expr_node_ptr dup_of_byExpr;

  DBUG_ENTER("replaceAllOccurencesAndSetExpansiondepth");
  DBUG_ASSERT(ofVar            != NULL);
  DBUG_ASSERT(parent_of_inExpr != NULL);
  DBUG_ASSERT(inExpr           != NULL);

  if ( (inExpr->nodetype == e_variable) && (inExpr->strrep != NULL)
       && (0 == strcmp(ofVar, inExpr->strrep)) ) {
    //each replacement needs a real copy of the byExpr
    dup_of_byExpr = duplicateSubtreeButSetNewExpansiondepth(byExpr, 
							    expansiondepth);
    DBUG_ASSERT(dup_of_byExpr != NULL);
    replaceSubtree(parent_of_inExpr, inExpr, dup_of_byExpr);
  } else {
    
    // visibility for let statements
    if ( inExpr->nodetype == e_let ) {
      // replace all occurences in the expression-part only if the 
      // variable is not defined in the let 
      Expr_node_ptr vars = LET_VARS_EXPR_NODE(inExpr);
      int variable_is_defined_here = 0;
      while ( vars != NULL ) {
        if ( 0 == strcmp(ARG_EXPR_NODE(vars)->strrep,ofVar) ) {
             variable_is_defined_here = 1;
             vars = NULL;
        } else {
          vars = NEXT_EXPR_NODE(vars);
        }
      }
      if ( ! variable_is_defined_here ) {
       
        replaceAllOccurencesAndSetExpansiondepth(ofVar, inExpr, 
                                               LET_EXPR_EXPR_NODE(inExpr), byExpr,
                                               expansiondepth);
      }
      // always replace all occurences in the value-part
      replaceAllOccurencesAndSetExpansiondepth(ofVar, inExpr, 
                                               LET_VAL_EXPR_NODE(inExpr), byExpr,
                                               expansiondepth);
      // never replace the variable-definition-part
    } else {
      
      for (i=0;  i < EXPR_NODE_MAX_CHILDREN;  i++) {
        if (inExpr->child[i] != NULL) {
	     replaceAllOccurencesAndSetExpansiondepth(ofVar, inExpr, 
						 inExpr->child[i], byExpr,
						 expansiondepth);
        }
      } //for i
    }
    
  } //if == e_variable

  DBUG_VOID_RETURN;
} //replaceAllOccurencesAndSetExpansiondepth


/* @return FALSE if the patternmatching failed and TRUE otherwise
 */
gboolean 
expandThisFunctioncall(Expr_node_ptr parent_of_node, Expr_node_ptr node)
{
  gboolean res = TRUE;
  Symboltable_entry_ptr pSte;
  Fun_node_ptr pFun;
  Expr_node_ptr right_body_clause, arg_list, new_body;
  GList *variable_list, *act_variable, *replacement_list, *act_replacement;
  gint new_expansiondepth, saved_index;

  DBUG_ENTER("expandThisFunctioncall");
  DBUG_ASSERT(node != NULL);
  DBUG_ASSERT(node->nodetype == e_userfun);

  pSte = seekSymbolId(symbol_table, node->strrep);
  DBUG_ASSERT(pSte != NULL);
  pFun = pSte->definition_point;
  arg_list = ARGLIST_EXPR_NODE(node);


  /* do patternmatching to get the right clauses
   * - has to build the variable_list    (in form of a GList*)
   * - has to build the replacement_list (in form of a GList*)
   */
  gcInitializeBySize(8); /* patternMatching might create some additional
			  * e_listkeeper's */

  if(!patternMatching(pFun, arg_list, &right_body_clause, 
		      &variable_list, &replacement_list)) {
    res = FALSE;
  } else {

    new_expansiondepth = node->expansiondepth +1;
    eval_stats.expansiondepth = new_expansiondepth;
    eval_stats.max_reached_expansiondepth = 
      MAX(eval_stats.max_reached_expansiondepth, new_expansiondepth);
    /* replace function-call by right_body_clause */
    new_body = duplicateSubtreeButSetNewExpansiondepth(right_body_clause,
						       new_expansiondepth);
    //avoid that arg_list will be freed when replacing subtree
    //because it also contains the items of replacement_list
    ARGLIST_EXPR_NODE(node) = NULL;
    replaceSubtree(parent_of_node, node, new_body);
    eval_stats.nr_expansions++;


    /* replace the variables within the new_body with the correct arguments
     */
    if ((variable_list != NULL) && (replacement_list != NULL)) {
      act_variable    = variable_list;
      act_replacement = replacement_list;
      while ((act_variable != NULL) && (act_replacement != NULL)) {
	saved_index = getIndexOfChild(parent_of_node, new_body);
	replaceAllOccurencesAndSetExpansiondepth(act_variable->data, 
						 parent_of_node, 
						 new_body, 
						 act_replacement->data, 
						 new_expansiondepth);
	new_body = parent_of_node->child[saved_index];
	act_variable    = g_list_next(act_variable);
	act_replacement = g_list_next(act_replacement);
      } //while
      DBUG_ASSERT(act_variable    == NULL);
      DBUG_ASSERT(act_replacement == NULL);
      my_g_list_free_except_items(variable_list); //do not free the items
      my_g_list_free_except_items(replacement_list); //items are freed later
    } //if ((variable_list != NULL) && (replacement_list != NULL))


    /* now free the arg_list */
    free_expr_node(arg_list); //also frees the items of the replacement_list
  } //if patternMatching failed (runtimeerror)
  gcFinalize(); //frees all additional e_listkeeper's

  DBUG_PRINT("eval", ("res == %d", res));
  DBUG_RETURN(res);
} //expandThisFunctioncall



/* @return 1 if at least one expansion was possible (it may have faild or 
             succeeded)
 *         0 of no expansion was possible
 */
gint 
expand_FULL(Expr_node_ptr parent_of_node, Expr_node_ptr node)
{
  gboolean tmp, res = 0;
  gint i;

  DBUG_ENTER("expand_FULL");

  for (i=0;  i < EXPR_NODE_MAX_CHILDREN;  i++) {
    if (node->child[i] != NULL) {
      tmp = expand_FULL(node, node->child[i]);
      if ((res == 0) && (tmp == 0)) {
	res = 0;
      } else {
	res = 1;
      } //if-else
    } //if
  } //for i
  
  if (node->nodetype == e_userfun) {
    expandThisFunctioncall(parent_of_node, node);
    res = 1;
  } //if (node->nodetype == e_userfun)
  
  DBUG_PRINT("eval", ("res == %d", res));
  DBUG_RETURN(res);
} //expand_FULL
