#include <stdio.h>
#include <string.h>
#include <gtk/gtk.h>
#include "../Misc/my_dbug.h"
#include "../Misc/error_interface.h"
#include "../Syntaxtree/syntaxtree.h"
#include "../Typechecker/tc_util.h"
#include "eval_global.h"
#include "resources.h"

Eval_statistics eval_stats = {0, 0, 0, 0};
void (*fpUpdateStatistics) (void) = NULL;


void 
runtime_error(gchar* msg)
{
  const gint c_max = 48;
  gchar line0[c_max], line1[c_max], line2[c_max], line3[c_max], line4[c_max], 
        line5[c_max];

  DBUG_ENTER("runtime_error");

  sprintf(line0, "Runtimeerror");
  sprintf(line1, "-------------");
  sprintf(line2, "%s", msg);
  sprintf(line3, " ");
  sprintf(line4, " ");
  sprintf(line5, " ");

  //check afterwards, if local variables lineX were big enough
  DBUG_ASSERT(strlen(line0) < c_max);
  DBUG_ASSERT(strlen(line1) < c_max);
  DBUG_ASSERT(strlen(line2) < c_max);
  DBUG_ASSERT(strlen(line3) < c_max);
  DBUG_ASSERT(strlen(line4) < c_max);
  DBUG_ASSERT(strlen(line5) < c_max);

  set_ErrorDescription(1, BUTTON_WARNUNG_OK_TEXT, line0, line1,
                       line2, line3, line4, line5);

  DBUG_VOID_RETURN;
} //runtime_error


gboolean 
areMLValuesEqual(Expr_node_ptr n1, Expr_node_ptr n2)
{
  gboolean res;

  DBUG_ENTER("areMLValuesEqual");

  if (n1->nodetype != n2->nodetype) {
    res = FALSE;
  } else {
    if        (n1->valuetype == unit_type_index) {
      res = (n1->value.mlunit == n2->value.mlunit);

    } else if (n1->valuetype == bool_type_index) {
      res = (n1->value.mlbool == n2->value.mlbool);

    } else if (n1->valuetype == int_type_index) {
      res = (n1->value.mlint == n2->value.mlint);

    } else if (n1->valuetype == real_type_index) {
      res = (n1->value.mlreal == n2->value.mlreal);

    } else if (n1->valuetype == string_type_index) {
      res = ( (n1->value.mlstring) && (n2->value.mlstring) &&
	      (0 == strcmp(n1->value.mlstring, n2->value.mlstring)) );

    } else if (n1->valuetype == nil_type_index) {
      res = TRUE;

    } else if (n1->valuetype == list_type_index) {
      res = TRUE;

    } else {
      res = FALSE; //to avoid a compiler warning
      DBUG_ASSERT(0);
    } //if-switch: valuetype
  } //if different nodetypes

  DBUG_RETURN(res);
} //areMLValuesEqual


void 
my_g_list_free_except_items(GList* list)
{
  GList* tmp;
  DBUG_ENTER("my_g_list_free_except_items");

  tmp = list;
  while (tmp != NULL) {
    tmp->data = NULL;
    tmp = g_list_next(tmp);
  } //while

  g_list_free(list);

  DBUG_VOID_RETURN;
} //my_g_list_free_except_items


