#include <gtk/gtk.h>
#include "../Misc/my_dbug.h"
#include "../Misc/mem_management.h" 
#include "../Syntaxtree/syntaxtree.h"
#include "eval_simplify.h"
#include "eval_expand.h"


/* @return >= 0 number of executed evaluations
 *          -1  an runtime-error occurred
 */
static gint 
eval_LMIM(Expr_node_ptr parent_of_node, Expr_node_ptr node,
	  gint maxNrEvaluations, gboolean simplify_only)
{
  gint i, saved_index, nrEvals = 0, res = 0;
  gboolean leave;

  DBUG_ENTER("eval_LMIM");
  DBUG_ASSERT(node != NULL);

  if ((maxNrEvaluations > 0) && (eval_stats.userbreak == FALSE)) {

    if ( (node->nodetype == e_ifthenelse) || 
	 (node->nodetype == e_andalso)    ||
	 (node->nodetype == e_orelse) ) {
      //visit only first child
      nrEvals = eval_LMIM(node, node->child[0], maxNrEvaluations, 
			  simplify_only);
      if (nrEvals == -1) {
	res = -1;
      } else {
	res              += nrEvals;
	maxNrEvaluations -= nrEvals;
      } //if-else
  } else if (  (node->nodetype == e_let) ) {
      //visit only second child
    nrEvals = eval_LMIM(node, node->child[1], maxNrEvaluations,                             simplify_only);
        if (nrEvals == -1) {
          res = -1;
        } else {
          res              += nrEvals;
          maxNrEvaluations -= nrEvals;
        } //if-else
   } else {
      //visit all children
      for ((i=0, leave=FALSE);  (i < EXPR_NODE_MAX_CHILDREN) && !leave;  i++) {
	DBUG_PRINT("eval", ("i=%d;  leave=%d", i, leave));
	if (node->child[i] != NULL) {
	  nrEvals = eval_LMIM(node, node->child[i], maxNrEvaluations, 
			      simplify_only);
	  if (nrEvals == -1) {
	    leave = TRUE;
	    res = -1;
	  } else {
	    res              += nrEvals;
	    maxNrEvaluations -= nrEvals;
	    if (maxNrEvaluations <= 0) {
	      leave = TRUE;
	    } //if
	  } //if-else
	} //if node->child[i] != NULL
      } //for i
      DBUG_PRINT("eval", ("after for i=%d;  leave=%d", i, leave));
    } //if if or and or or

    //work on this node
    leave = FALSE;
    saved_index = getIndexOfChild(parent_of_node, node);
    /*
    DBUG_PRINT("eval2", ("parent_of_node==%x   node==%x", 
			 parent_of_node, node));
    */
    if (saved_index == -1) {
      printf_expr_node(parent_of_node);
      printf_expr_node(node);
    }
    DBUG_ASSERT(saved_index >= 0);
    if ((res != -1) && (maxNrEvaluations > 0)) {
      nrEvals = evalThisNode(parent_of_node, node, e_lmim, simplify_only);
      node = parent_of_node->child[saved_index];
      if (nrEvals == -1) {
	res = -1;
	//leave = TRUE;
      } else {
	if (nrEvals == 0) {
	  leave = TRUE;
	} else {
	  res              += nrEvals;
	  maxNrEvaluations -= nrEvals;
	} //if-else
      } //if-else
    } //if maxNrEvaluations > 0
    while ((res != -1) && (maxNrEvaluations > 0) && !leave) {
      nrEvals = eval_LMIM(parent_of_node, node, maxNrEvaluations, 
			  simplify_only);
      node = parent_of_node->child[saved_index];
      if (nrEvals == -1) {
	res = -1;
	//leave = TRUE;
      } else {
	if (nrEvals == 0) {
	  leave = TRUE;
	} else {
	  res              += nrEvals;
	  maxNrEvaluations -= nrEvals;
	} //if-else
      } //if-else
    } //while (res != -1) && (maxNrEvaluations > 0)
    
  } //if maxNrEvaluations > 0

  DBUG_RETURN(res);
} //eval_LMIM


/* @return >= 0 number of executed evaluations
 *          -1  an runtime-error occurred
 */
static gint 
eval_LMOM(Expr_node_ptr parent_of_node, Expr_node_ptr node,
	  gint maxNrEvaluations, gboolean simplify_only)
{
  gint i, nrEvals = 0, res = 0;
  gboolean leave;

  DBUG_ENTER("eval_LMOM");
  DBUG_ASSERT(node != NULL);

  if ((maxNrEvaluations > 0)  && (eval_stats.userbreak == FALSE)) {
    if ((node->nodetype == e_userfun) && !simplify_only) {

      leave = FALSE;
      while ((maxNrEvaluations > 0) && !leave) {
	if (expandThisFunctioncall(parent_of_node, node)) {
	  leave = TRUE;
	  res = 1;
	} else {
	  nrEvals = eval_LMOM(node, ARGLIST_EXPR_NODE(node), 1, simplify_only);
	  if (nrEvals <= 0) {
	    leave = TRUE;
	    res = -1;
	  } else {
	    res              += nrEvals;
	    maxNrEvaluations -= nrEvals;
	  } //if-else
	} //if-else expandThisFunctioncall
      } //while

    } else {

      if ( (node->nodetype == e_ifthenelse) || 
	   (node->nodetype == e_andalso)    ||
	   (node->nodetype == e_orelse) ) {

	//visit only first child
	nrEvals = eval_LMOM(node, node->child[0], maxNrEvaluations, 
			    simplify_only);
	if (nrEvals == -1) {
	  res = -1;
	} else {
	  res              += nrEvals;
	  maxNrEvaluations -= nrEvals;
	} //if-else

    } else if ( (node->nodetype == e_let) ) {
      //visit only second child
      nrEvals = 0;//eval_LMOM(node, node->child[1], maxNrEvaluations, simplify_only);
      if (nrEvals == -1) {
        res = -1;
      } else {
        res              += nrEvals;
        maxNrEvaluations -= nrEvals;
      } //if-else

    } else {
	//visit the children
	for (i=0, leave=FALSE;  (i < EXPR_NODE_MAX_CHILDREN) && !leave;  i++) {
	  if (node->child[i] != NULL) {
	    nrEvals = eval_LMOM(node, node->child[i], maxNrEvaluations, 
				simplify_only);
	    if (nrEvals == -1) {
	      leave = TRUE;
	      res = -1;
	    } else {
	      if (nrEvals > 0) {
		res              += nrEvals;
		maxNrEvaluations -= nrEvals;
		leave = TRUE;
	      } //if
	    } //if-else
	  } //if node->child[i] != NULL
	} //for i
      } //if if or and or or

      //work on this node
      if ((res != -1)  && (maxNrEvaluations > 0)) {
        
	nrEvals = evalThisNode(parent_of_node, node, e_lmom, simplify_only);
	if (nrEvals == -1) {
	  res = -1;
	} else {
	  res += nrEvals;
	} //if-else
      } //if (res != -1)
    } //if (node->nodetype == e_userfun) && !simplify_only)
  } //if maxNrEvaluations > 0

  DBUG_RETURN(res);
} //eval_LMOM


/* @return FALSE in case of an runtimeerror or TRUE otherwise
 */
gboolean 
evaluate(Expr_node_ptr parent_of_node, Expr_node_ptr node, EvalMode mode,
	 gint maxNrEvaluations, gboolean simplify_only)
{
  gboolean res  = TRUE, 
           stop = FALSE;
  gint err, saved_stats, tmp, saved_index;
  //  gchar* tmpStr;

  DBUG_ENTER("evaluate");

  eval_stats.userbreak = FALSE;
  DBUG_PRINT("eval", ("EvalMode == %d", mode));
  if (mode == e_lmim) {

    while ( (stop == FALSE) && (eval_stats.userbreak == FALSE) ) {
      saved_index = getIndexOfChild(parent_of_node, node);
      DBUG_ASSERT(saved_index >= 0);
      err = eval_LMIM(parent_of_node, node, maxNrEvaluations, simplify_only);
      node = parent_of_node->child[saved_index];
      if (err == -1) {
	stop = TRUE;
	res  = FALSE;
      } else {
	maxNrEvaluations -= err;
	if ((err == 0) || (maxNrEvaluations <= 0)) {
	  stop = TRUE;
	} //if
      } //if-else
    } //while

  } else if (mode == e_lmom) {

    while ( (stop == FALSE) && (eval_stats.userbreak == FALSE) ) {
      saved_index = getIndexOfChild(parent_of_node, node);
      DBUG_ASSERT(saved_index >= 0);
      err = eval_LMOM(parent_of_node, node, maxNrEvaluations, simplify_only);
      node = parent_of_node->child[saved_index];
      if (err == -1) {
	stop = TRUE;
	res  = FALSE;
      } else {
	maxNrEvaluations -= err;
	if ((err == 0) || (maxNrEvaluations <= 0)) {
	  stop = TRUE;
	} //if
      } //if-else
    } //while

  } else if (mode == e_full) {

    while ( (stop == FALSE) && (eval_stats.userbreak == FALSE) ) {
      saved_index = getIndexOfChild(parent_of_node, node);
      DBUG_ASSERT(saved_index >= 0);
      err = eval_LMIM(parent_of_node, node, maxNrEvaluations, TRUE);
      node = parent_of_node->child[saved_index];
      DBUG_PRINT("eval2", ("eval_LMIM(...) == %d\tmaxNrEvaluations == %d", 
			   err, maxNrEvaluations));
      if ((err != -1) && ((maxNrEvaluations -= err) > 0)) {
	saved_stats = eval_stats.nr_expansions;
	saved_index = getIndexOfChild(parent_of_node, node);
	DBUG_ASSERT(saved_index >= 0);
	tmp = expand_FULL(parent_of_node, node);
	node = parent_of_node->child[saved_index];
	DBUG_PRINT("eval2", ("expand_FULL(...) == %d", tmp));
	if (tmp == 0) {
	  stop = TRUE;
	  //res = TRUE;
	} else if ( (tmp == 1) && 
		    (0 == (eval_stats.nr_expansions - saved_stats)) ) {
	  res  = FALSE;
	  stop = TRUE;
	} else {
	  maxNrEvaluations--;
	  //res = TRUE;
	} //if-switch
      } else {
	res  = (err != -1);
	stop = TRUE;
      }//if-else
    } //while

  } else {
    DBUG_ASSERT(0);
    res = FALSE; //only to avoid a compiler warning
  } //if-switch

  if (fpUpdateStatistics != NULL) {
    (*fpUpdateStatistics)();
  }

  DBUG_PRINT("eval", ("res == %d", res));
  DBUG_RETURN(res);
} //evaluate



