#include<gtk/gtk.h>
#include "../Misc/my_dbug.h"
#include "../Misc/mem_management.h"
#include "../Misc/utilities.h"
#include "../Typechecker/tc_util.h"
#include "kiel_global.h" //BAUMFENSTER_TYP
#include "kiel_ressource.h" //K_KnotenBreite_zu_SchriftBreite
#include "expr_node_misc.h"
#include "undoredo.h"
#include "drawable_expr.h"


typedef struct _WethSha_parameters {
  /* 
   * +---+                   +---+
   * | x |<---min_delta_x--->| y |
   * +---+                   +---+
   *   ^
   *   |
   * delta_y
   *   |
   *   v
   * +---+
   * | z |
   * +---+
   */
  gint min_delta_x;
  gint delta_y; 
  gint DEN_height; //height of a Drawable_Expr_Node in pixel
} WethSha_parameters_struct;


static WethSha_parameters_struct wethSha_params;
static gint max_x; //set by wethShaModifyingPass
static gint max_y; //set by createMisc_prepare_getHeight


Drawable_expr_ptr 
create_drawable_expr(Expr_node_ptr pAct_expr)
{
  Drawable_expr_ptr res;

  DBUG_ENTER("create_drawable_expr");
  res = mtmalloc(sizeof(Drawable_expr_struct));

  res->expr_height   = -1;
  res->initial_expr  = pAct_expr;
  res->act_expr      = pAct_expr;
  res->redo_list     = NULL;
  res->undo_list     = NULL;
  res->drawingAllowed = TRUE;

  DBUG_RETURN(res);
} //create_drawable_expr


void 
free_drawable_expr(Drawable_expr_ptr pDrawExpr)
{
  DBUG_ENTER("free_Drawable_expr");

  if (pDrawExpr != NULL) {
    //free act_expr, undo_list, redo_list
    pDrawExpr->initial_expr = NULL; //it is only an additional pointer
    free_expr_node_inclusiv_misc(pDrawExpr->act_expr);
    free_undoredo_list(pDrawExpr->undo_list);
    pDrawExpr->undo_list = NULL;
    free_undoredo_list(pDrawExpr->redo_list);
    pDrawExpr->redo_list = NULL;

    //free itself
    mtfree(pDrawExpr);
  } //if

  DBUG_VOID_RETURN;
} //free_Drawable_expr


void 
set_WethSha_parameters(gint min_delta_x, gint delta_y, gint DEN_height)
{
  DBUG_ENTER("set_WethSha_parameters");

  wethSha_params.min_delta_x = min_delta_x;
  wethSha_params.delta_y     = delta_y;
  wethSha_params.DEN_height  = DEN_height;

  DBUG_VOID_RETURN;
} //set_WethSha_parameters


static gchar* 
getExprText(Expr_node_ptr pExpr)
{
  gchar* res;
  Expr_node_misc_ptr pMisc;

  DBUG_ENTER("getExprText");
  DBUG_ASSERT(pExpr != NULL);

  pMisc = pExpr->misc;
  if ( (pExpr->nodetype == e_constant) && (pMisc != NULL) &&
       (pExpr->valuetype == string_type_index) ) {
    res = pMisc->string_strrep;
  } else {
    res = pExpr->strrep;
  } //if
  if (res == NULL) {
    res = "?";
  }

  DBUG_RETURN(res);
} //getExprText


gint 
getExprPixelmiddle_x(Expr_node_ptr pExpr)
{
  gint res = 0;
  Expr_node_misc_ptr pMisc;

  DBUG_ENTER("getExprPixelmiddle_x");
  DBUG_ASSERT(pExpr != NULL);

  pMisc = pExpr->misc;
  if (pMisc != NULL) {
    res = pMisc->x;
  } //if

  DBUG_RETURN(res);
} //getExprPixelmiddle_x


gint 
getExprPixelmiddle_y(Expr_node_ptr pExpr)
{
  gint res = 0;
  Expr_node_misc_ptr pMisc;

  DBUG_ENTER("getExprPixelmiddle_y");
  DBUG_ASSERT(pExpr != NULL);

  pMisc = pExpr->misc;
  if (pMisc != NULL) {
    res = pMisc->y + (wethSha_params.DEN_height / 2);
  } //if

  DBUG_RETURN(res);
} //getExprPixelmiddle_y


static gint 
getExprPixelwidth(Expr_node_ptr pExpr, BAUMFENSTER_TYP bmf)
{
  gint res;
  gchar* string;

  DBUG_ENTER("getExprPixelwidth");
  DBUG_ASSERT(pExpr != NULL);

  string = getExprText(pExpr);
  res = (8 + gdk_string_width(bmf->pix_draw->draw_a->style->font, string)) * 
        K_KnotenBreite_zu_SchriftBreite;

  DBUG_RETURN(res);
} //getExprPixelwidth


gint 
getExprPixelheight(Expr_node_ptr pExpr, BAUMFENSTER_TYP bmf)
{
  gint res;
  gchar* string;

  DBUG_ENTER("getExprPixelheight");
  DBUG_ASSERT(pExpr != NULL);

  string = getExprText(pExpr);
  res = gdk_string_height(bmf->pix_draw->draw_a->style->font, "8") *
        K_KnotenHoehe_zu_SchriftHoehe;

  DBUG_RETURN(res);
} //getExprPixelheight


static gint 
getTotalPixelwidth(void)
{
  DBUG_ENTER("getTotalPixelwidth");
  DBUG_RETURN(max_x + wethSha_params.min_delta_x);
} //getTotalPixelwidth


static gint 
getTotalPixelheight(void)
{
  DBUG_ENTER("getTotalPixelheight");
  DBUG_RETURN(max_y + wethSha_params.delta_y);
} //getTotalPixelheight


/* @return true if the node has to be drawn and false if it is only a 
 *         helpful node which must not be drawn
 */
gboolean 
isDrawableNode(Expr_node_ptr pExpr)
{
  gboolean res;

  DBUG_ENTER("isDrawableNode");
  DBUG_ASSERT(pExpr != NULL);

  res = !(
	  (pExpr->nodetype == e_list)       || 
          (pExpr->nodetype == e_letvars_list)     ||
          (pExpr->nodetype == e_arg_list) 
        );

  DBUG_RETURN(res);
} //isDrawableNode


static gboolean 
isParentOfNotDrawableNode(Expr_node_ptr pExpr)
{
  gboolean res;

  DBUG_ENTER("isParentOfNotDrawableNode");
  DBUG_ASSERT(pExpr != NULL);

  res = (pExpr->nodetype == e_listkeeper)       || 
        (pExpr->nodetype == e_constructor)      || 
	(pExpr->nodetype == e_let)      || 
	
        (pExpr->nodetype == e_userfun);

  DBUG_RETURN(res);
} //isParentOfNotDrawableNode


/* Creates the misc-struct of a Expr_node, sets the y within the misc-structure
 * and returns the (drawable) height of the expr.
 * @return height of the given pExpr (height of a leaf is 0)
 */
static gint
createMisc_prepare_getHeight(Expr_node_ptr pExpr, gint actLevel)
{
  gint res;
  gint i, hoc/*height of child*/, max_height, level_inc;
  Expr_node_misc_ptr pMisc;

  DBUG_ENTER("createMisc_prepare_getHeight");
  DBUG_ASSERT(pExpr != NULL);

  //work on this node
  if (pExpr->misc == NULL) {
    insert_new_expr_node_misc(pExpr);
  } 
  pMisc = pExpr->misc;
  pMisc->y = wethSha_params.delta_y + 
    actLevel * (wethSha_params.delta_y/*+ wethSha_params.DEN_height*/);
  max_y = MAX(max_y, pMisc->y);
  if (!isDrawableNode(pExpr)) {
    pMisc->y -= 17; //debug only
  } //if
  if (pExpr->nodetype == e_constant) {
    if (pExpr->valuetype == int_type_index) {
      strreplacechar(pExpr->strrep, '-', '~');
    } else if (pExpr->valuetype == real_type_index) {
      gchar tmp_str[MAXIMALE_NACHKOMMASTELLEN + 8];

      sprintf(tmp_str, "%.*e", pMisc->max_drawn_digits, pExpr->value.mlreal);
      strreplacechar(tmp_str, '-', '~');
      mtfree(pExpr->strrep);
      pExpr->strrep = mtstrdup(tmp_str);
    } else if (pExpr->valuetype == string_type_index) {
      gchar* tmp_str;
      gint len;

      len = MIN(strlen(pExpr->strrep), pMisc->max_drawn_digits);
      tmp_str = mtmalloc(sizeof(gchar) * (len + 1));
      strncpy(tmp_str, pExpr->strrep, len);
      mtfree(pMisc->string_strrep);
      pMisc->string_strrep = mtstrconcat(3, "\"", tmp_str, "\"");
      mtfree(tmp_str);
    } //if-switch
  } else {
  } //if e_constant

  /*  now done in create_expr_...()
  if        ((pExpr->nodetype == e_constant) && (pExpr->strrep == NULL)) {
    pExpr->strrep = mlConstant2String(pExpr);
  } else if ((pExpr->nodetype == e_nil_const) && (pExpr->strrep == NULL)) {
    pExpr->strrep = mtstrdup("[]");
  }
  */

  //visit children and get their (drawable) height
  max_height = 0;
  if (isDrawableNode(pExpr)) {
    level_inc = 1;
  } else {
    level_inc = 0;
  } //if
  for (i=0;  i < EXPR_NODE_MAX_CHILDREN;  i++) {
    if (pExpr->child[i] != NULL) {
      hoc = createMisc_prepare_getHeight(pExpr->child[i], actLevel +level_inc);
      max_height = MAX(max_height, hoc);
    } //if
  } //for i
  res = max_height +1;

  DBUG_RETURN(res);
} //createMisc_prepare_getHeight


void
prepareForWethSha(Drawable_expr_ptr pDrawExpr)
{
  DBUG_ENTER("prepareForWethSha");

  max_y = 0;
  pDrawExpr->expr_height = createMisc_prepare_getHeight(pDrawExpr->act_expr,0);

  DBUG_VOID_RETURN;
} //prepareForWethSha


/* Calculates the optimal x-position of a node between its children.
 * @return optimal x-position for center of node
 */
static gint 
calculatePlace(Expr_node_ptr pExpr, BAUMFENSTER_TYP bmf)
{
  gint res = 0,
       nr_children = 0, i;
  Expr_node_ptr      pNode;
  Expr_node_misc_ptr pMisc;

  DBUG_ENTER("calculatePlace");
  DBUG_ASSERT(pExpr != NULL);

  //visit children
  if (isParentOfNotDrawableNode(pExpr)) {

    pNode = pExpr->child[0];
    while (pNode != NULL) {
      nr_children++;
      pMisc = (pNode->child[0])->misc;
      res += pMisc->x;
      pNode = pNode->child[1];
    } //while

  } else {

    for (i=0;  i < EXPR_NODE_MAX_CHILDREN;  i++) {
	if (pExpr->child[i] != NULL && pExpr->child[i]->subnodetype != e_hidden ) {
	nr_children++;
	pMisc = (pExpr->child[i])->misc;
	res += pMisc->x;
      } //if
    } //for i

  } //if isParentOfNotDrawableNode
  
  res =  res / nr_children;

  DBUG_RETURN(res);
} //calculatePlace


static void 
wethShaFirstPass(Expr_node_ptr pExpr, gint* next_lx, gint* modifier,
		 gint h, BAUMFENSTER_TYP bmf)
{
  gint place, i, h_inc, halfPixelwidth;
  gint next_x; // Next free x-position for the center of node
  gboolean is_leaf = TRUE;
  Expr_node_misc_ptr pMisc, pChildMisc;

  DBUG_ENTER("wethShaFirstPass");

  if (pExpr != NULL) {
    //visit children
    if (isDrawableNode(pExpr)) {
      h_inc = 1;
    } else {
      h_inc = 0;
    } //if
    for (i=0;  i < EXPR_NODE_MAX_CHILDREN;  i++) {
	if (pExpr->child[i] != NULL && pExpr->child[i]->subnodetype != e_hidden) {
	is_leaf = FALSE;
	wethShaFirstPass(pExpr->child[i], next_lx, modifier, h + h_inc, bmf);
      } //if
    } //for i

    //work on this node
    pMisc = pExpr->misc;
    if (!isDrawableNode(pExpr)) {
	pMisc->wethSha_modifier = 0;
	pChildMisc = pExpr->child[0]->misc;
	pMisc->x   = pChildMisc->x;
    } else {
      halfPixelwidth = getExprPixelwidth(pExpr, bmf) / 2;
      next_x         = next_lx[h] + halfPixelwidth;//get center from given left
      if (is_leaf) {
	pMisc->x = next_x;
      } else {
	place = calculatePlace(pExpr, bmf);
	if (next_x <= (place + modifier[h])) {
	  pMisc->x = place + modifier[h];
	} else {
	  pMisc->x    = next_x;
	  modifier[h] = next_x - place;
	} //if
	pMisc->wethSha_modifier = modifier[h];
      } //if is_leaf
      next_lx[h] = pMisc->x + halfPixelwidth + //right border
	           wethSha_params.min_delta_x; //plus min_delta_x
      /* pMisc->y is already set by 
       * 'exprNode2drawableExpr_and_prepareForWethSha()' */
    } //if isDrawableNode
  } //if pExpr != NULL

  DBUG_VOID_RETURN;
} //wethShaFirstPass


static void 
wethShaModifyingPass(Expr_node_ptr pExpr, gint modifier_sum, 
		     BAUMFENSTER_TYP bmf)
{
  gint i;
  Expr_node_misc_ptr pMisc;

  DBUG_ENTER("wethShaModifyingPass");

  /* The wethSha_modifier of an not drawable node has to be 0! */
  if (pExpr != NULL) {
    //work on this node
    pMisc = pExpr->misc;
    pMisc->x += modifier_sum;
    max_x = MAX(max_x, pMisc->x + (getExprPixelwidth(pExpr, bmf) / 2));

    //visit children
    for (i=0;  i < EXPR_NODE_MAX_CHILDREN;  i++) {
      if (pExpr->child[i] != NULL) {
	wethShaModifyingPass(pExpr->child[i], 
			     modifier_sum + pMisc->wethSha_modifier, bmf);
      } //if
    } //for i
  } //if pExpr != NULL

  DBUG_VOID_RETURN;
} //wethShaModifyingPass


/* Slightly modified algorithm of Wetherell and Shannon to position
 * the nodes of a tree on screen.
 */
void 
wetherellShannon(Drawable_expr_ptr pDrawExpr, BAUMFENSTER_TYP bmf)
{
  gint* next_lx; /* stores the next free X-position where the Left border
		  * of a node may start */
  gint* modifier;
  gint i;

  DBUG_ENTER("wetherellShannon");

  //init arrays
  next_lx  = mtmalloc((pDrawExpr->expr_height +1) * sizeof(gint));
  modifier = mtmalloc((pDrawExpr->expr_height +1) * sizeof(gint));
  for (i=0;  i <= pDrawExpr->expr_height; i++) {
    next_lx[i]  = wethSha_params.min_delta_x;
    modifier[i] = 0;
  } //for i

  //position nodes
  wethShaFirstPass    (pDrawExpr->act_expr, next_lx, modifier, 0, bmf);
  max_x = 0;
  wethShaModifyingPass(pDrawExpr->act_expr, 0, bmf);

  pDrawExpr->totalPixelwidth  = getTotalPixelwidth ();
  pDrawExpr->totalPixelheight = getTotalPixelheight();

  //free arrays
  mtfree(next_lx);
  mtfree(modifier);

  DBUG_VOID_RETURN;
} //wetherellShannon


static void 
drawEdge(Expr_node_ptr pParent, Expr_node_ptr pChild, BAUMFENSTER_TYP bmf)
{
  gint x1, y1, x2, y2;
  Expr_node_misc_ptr pMiscParent, pMiscChild;

  DBUG_ENTER("drawEdge");
  DBUG_ASSERT(pParent != NULL);
  DBUG_ASSERT(pChild  != NULL);

  pMiscParent = pParent->misc;
  pMiscChild  = pChild->misc;
  if ((pMiscParent != NULL) && (pMiscChild != NULL)) {
    x1 = pMiscParent->x - bmf->adjustmenth->value;
    y1 = pMiscParent->y - bmf->adjustmentv->value;
    x2 = pMiscChild->x  - bmf->adjustmenth->value;
    y2 = pMiscChild->y + getExprPixelheight(pParent, bmf)
         - bmf->adjustmentv->value;
    gdk_draw_line(bmf->pix_draw->pixm,
		  bmf->pix_draw->draw_a->style->black_gc,
		  x1,y1,
		  x2,y2);
  } //if

  DBUG_VOID_RETURN;
} //drawEdge

static void 
drawSingleNode(Expr_node_ptr pExpr, gboolean thick, BAUMFENSTER_TYP bmf)
{
  gchar* string;
  gint lx, //Left-border X-position
       y,  //top-border Y-position
       pixelWidth; 
  Expr_node_misc_ptr pMisc;

  DBUG_ENTER("drawSingleNode");
  DBUG_ASSERT(pExpr != NULL);

  pMisc = pExpr->misc;
  if ( (bmf->pix_draw->pixm) && (pMisc != NULL) ) {
    pixelWidth = getExprPixelwidth(pExpr, bmf);
    lx = pMisc->x - (pixelWidth / 2) - bmf->adjustmenth->value;
    y  = pMisc->y                    - bmf->adjustmentv->value;

    if (pExpr->nodetype == e_errornode){
      gdk_draw_rectangle(bmf->pix_draw->pixm, 
                         bmf->pix_draw->draw_a->style->fg_gc[2],
                         TRUE,
                         lx, y,
                         pixelWidth, wethSha_params.DEN_height);
      gdk_draw_rectangle(bmf->pix_draw->pixm, 
                         bmf->pix_draw->draw_a->style->black_gc,
                         FALSE,
                         lx, y,
                         pixelWidth, wethSha_params.DEN_height);
    } else {
      /* draw filled ellipse */
      if (pExpr->nodetype == e_constant) {
            gdk_draw_arc(bmf->pix_draw->pixm, 
                     bmf->pix_draw->draw_a->style->fg_gc[0],
                     TRUE,
                     lx, y,
                     pixelWidth, wethSha_params.DEN_height,
                     0*64, 360*64);  
      } else if (pExpr->nodetype == e_userfun) {
        gdk_draw_arc(bmf->pix_draw->pixm, 
                     bmf->pix_draw->draw_a->style->fg_gc[1],
                     TRUE,
                     lx, y,
                     pixelWidth, wethSha_params.DEN_height,
                     0*64, 360*64);
      } else if ( (pExpr->nodetype == e_constructor) ||
		  (pExpr->nodetype == e_listkeeper) ) {
        gdk_draw_arc(bmf->pix_draw->pixm, 
                     bmf->pix_draw->draw_a->style->fg_gc[2],
                     TRUE,
                     lx, y,
                     pixelWidth, wethSha_params.DEN_height,
                     0*64, 360*64);
      } //if-switch
      
     
      if (pExpr->subnodetype == e_boxed  ) {
        gdk_draw_rectangle(bmf->pix_draw->pixm, 
                           bmf->pix_draw->draw_a->style->black_gc,
                           FALSE,
                           lx, y,
                           pixelWidth, wethSha_params.DEN_height);
      } else {

      	/* draw not filled ellipse */
      gdk_draw_arc(bmf->pix_draw->pixm, 
                   bmf->pix_draw->draw_a->style->black_gc,
                   FALSE,
                   lx, y,
                   pixelWidth, wethSha_params.DEN_height,
                   0*64, 360*64);
      if (thick) {
	gdk_draw_arc(bmf->pix_draw->pixm, 
		     bmf->pix_draw->draw_a->style->black_gc,
		     FALSE,
		     lx+1, y+1,
		     pixelWidth-2, wethSha_params.DEN_height-2,
		     0*64, 360*64);
      } //if thick
      }
    } //if error_node
   
    /* text */
    string = getExprText(pExpr);
    gdk_draw_string(bmf->pix_draw->pixm,
                    bmf->pix_draw->draw_a->style->font,
                    bmf->pix_draw->draw_a->style->black_gc,
             lx+4+ (pixelWidth - 
	     (pixelWidth / K_KnotenBreite_zu_SchriftBreite)) * 0.5,
             y+ (wethSha_params.DEN_height - 
	     (wethSha_params.DEN_height / K_KnotenHoehe_zu_SchriftHoehe)) * 0.5
             + wethSha_params.DEN_height/K_KnotenHoehe_zu_SchriftHoehe,
                    string);
  } //if (bmf->pix_draw->pixm)

  DBUG_VOID_RETURN;
} //drawSingleNode


static void 
drawDrawableExprRec(Expr_node_ptr pExpr, Expr_node_ptr pParent,
		    Expr_node_ptr markedNode, BAUMFENSTER_TYP bmf)
{
  gint i;
  DBUG_ENTER("drawDrawableExprRec");
  DBUG_ASSERT(pExpr != NULL);

  //visit children
  for (i=0;  i < EXPR_NODE_MAX_CHILDREN;  i++) {
    if (pExpr->child[i] != NULL && pExpr->subnodetype != e_hidden ) {
      if (isDrawableNode(pExpr)) {
	drawDrawableExprRec(pExpr->child[i], pExpr, markedNode, bmf);
      } else {
	drawDrawableExprRec(pExpr->child[i], pParent, markedNode, bmf);
      }
    } //if
  } //for i

  //draw line from this node to its parent
  if (isDrawableNode(pExpr) && (pParent != NULL) && pExpr->subnodetype != e_hidden) {
    drawEdge(pExpr, pParent, bmf);
  } //if

  //draw this node
  if (isDrawableNode(pExpr) && pExpr->subnodetype != e_hidden) {
    drawSingleNode(pExpr, (pExpr == markedNode), bmf);
  } //if

  DBUG_VOID_RETURN;
} //drawDrawableExprRec


void 
drawDrawableExpr(Drawable_expr_ptr pDrawExpr, Expr_node_ptr markedNode, 
                 BAUMFENSTER_TYP bmf)
{
  DBUG_ENTER("drawDrawableExpr");
  DBUG_ASSERT(pDrawExpr != NULL);

  if ((pDrawExpr != NULL) && (pDrawExpr->drawingAllowed)) {
    drawDrawableExprRec(pDrawExpr->act_expr, NULL, markedNode, bmf);
  } //if

  DBUG_VOID_RETURN;
} //drawDrawableExpr


static void 
findClickedNodeAndParentHelp(Expr_node_ptr pExpr, gint mouse_x, gint mouse_y,
			     BAUMFENSTER_TYP bmf,Expr_node_ptr* refClickedNode,
			     Expr_node_ptr* refParent_of_node)
{
  gint i, half_pixelwidth, //pixelheight,
       lx, rx, ty, by;

  Expr_node_misc_ptr pMisc;

  DBUG_ENTER("findClickedNodeAndParentHelp");
  DBUG_ASSERT(*refClickedNode    == NULL);
  DBUG_ASSERT(*refParent_of_node == NULL);

  if (pExpr != NULL) {
    half_pixelwidth = getExprPixelwidth(pExpr, bmf) / 2;
    //pixelheight     = getExpePixelheight(pExpr, bmf);
    pMisc = pExpr->misc;
    lx = pMisc->x - half_pixelwidth;
    rx = pMisc->x + half_pixelwidth;
    ty = pMisc->y;
    by = pMisc->y + wethSha_params.DEN_height;
    if ( (lx <= mouse_x) && (mouse_x <= rx) &&
         (ty <= mouse_y) && (mouse_y <= by) ) {
       *refClickedNode    = pExpr;
       //*refParent_of_node = NULL; is allready NULL
    } //if

    for (i=0;  (*refClickedNode == NULL) && (i < EXPR_NODE_MAX_CHILDREN);  
	 i++) {
      findClickedNodeAndParentHelp(pExpr->child[i], mouse_x, mouse_y, bmf,
				   refClickedNode, refParent_of_node);
      if ((*refClickedNode != NULL) && (*refParent_of_node == NULL)) {
	*refParent_of_node = pExpr;
	i = EXPR_NODE_MAX_CHILDREN;
      } //if
    } //for i
  } //if

  DBUG_VOID_RETURN;
}  //findClickedNodeAndParentHelp

/* @param refClickedNode delivers the clicked node or null if none was hit
 * @param refParent_of_node delivers the parent of the clicked node or NULL
 *        if none was hit or the clicked node was the root
 */
void 
findClickedNodeAndParent(Expr_node_ptr root, gint mouse_x, gint mouse_y,
			 BAUMFENSTER_TYP bmf, Expr_node_ptr* refClickedNode, 
			 Expr_node_ptr* refParent_of_node)
{
  DBUG_ENTER("findClickedNodeAndParent");

  //guarantee both are NULL; is important to find the parent
  *refClickedNode    = NULL;
  *refParent_of_node = NULL;
  findClickedNodeAndParentHelp(root, mouse_x, mouse_y, bmf,
			       refClickedNode, refParent_of_node);
  DBUG_VOID_RETURN;
}  //findClickedNodeAndParent


static void 
calcPixelwidthAndHeightRec(Expr_node_ptr pExpr, BAUMFENSTER_TYP bmf)
{
  Expr_node_misc_ptr pMisc;
  gint i;

  DBUG_ENTER("calcPixelwidthAndHeightRec");

  if (pExpr != NULL) {
    //work on this node
    pMisc = pExpr->misc;
    max_x = MAX(max_x, pMisc->x + (getExprPixelwidth(pExpr, bmf) / 2));
    max_y = MAX(max_y, pMisc->y);

    //visit children
    for (i=0;  i < EXPR_NODE_MAX_CHILDREN;  i++) {
      if (pExpr->child[i] != NULL) {
	calcPixelwidthAndHeightRec(pExpr->child[i], 
				   bmf);
      } //if
    } //for i
  } //if pExpr != NULL

  DBUG_VOID_RETURN;
} //calcPixelwidthAndHeightRec


void 
calcPixelwidthAndHeight(Drawable_expr_ptr pDrawExpr, BAUMFENSTER_TYP bmf)
{
  DBUG_ENTER("calcPixelwidthAndHeight");

  max_x = 0;
  max_y = 0;
  calcPixelwidthAndHeightRec(pDrawExpr->act_expr, bmf);
  pDrawExpr->totalPixelwidth  = getTotalPixelwidth ();
  pDrawExpr->totalPixelheight = getTotalPixelheight();

  DBUG_VOID_RETURN;
} //calcPixelwidthAndHeight


