#include<gtk/gtk.h>
#include "../Misc/my_dbug.h"
#include "../Misc/error_interface.h"
#include "../Syntaxtree/syntaxtree.h"
#include "../Evaluator/eval_global.h"
#include "../Evaluator/evaluator.h"
#include "kiel_ressource.h"
#include "kiel_gtk.h"
#include "kiel_gtk_wf.h"
#include "kiel_gtk_awf.h"
#include "expr_node_misc.h"
#include "undoredo.h"
#include "drawable_expr.h"
#include "gtk_eval.h"



Expr_node_ptr eval_start_node_after_evaluation = NULL;



static void
update_statuslabel(void)
{
  gchar* label, int1[10], int2[10], int3[10], int4[10];

  DBUG_ENTER("update_statuslabel");

  sprintf(int1, "%d", eval_stats.expansiondepth);
  sprintf(int2, "%d", eval_stats.max_reached_expansiondepth);
  sprintf(int3, "%d", eval_stats.nr_expansions);
  sprintf(int4, "%d", eval_stats.nr_evaluations);

  label = mtstrconcat(15,
		      STATUSLABEL_EXPANSIONSTIEFE_TEXT, " ", 
		      int1, "   -   ", 
		      STATUSLABEL_MAX_EXPANSIONSTIEFE_TEXT, " ", 
		      int2, "   -   ", 
		      STATUSLABEL_EXPANSIONSANZAHL_TEXT,  " ", 
		      int3, "   -   ", 
		      STATUSLABEL_AUSWERTUNGSANZAHL_TEXT, " ", 
		      int4); 
  /*		      
  erg = g_strdup_printf("%s %3d   -   %s %3d   -   %s %3d   -   %s %4d",
		      STATUSLABEL_EXPANSIONSTIEFE_TEXT,   es.expansiondepth,
		      STATUSLABEL_MAX_EXPANSIONSTIEFE_TEXT, 
		      es.max_reached_expansiondepth,
		      STATUSLABEL_EXPANSIONSANZAHL_TEXT,  es.nr_expansions,
		      STATUSLABEL_AUSWERTUNGSANZAHL_TEXT, es.nr_evaluations);
  */
  setze_statuslabel(label);
  mtfree(label);

  DBUG_VOID_RETURN;
} //update_statuslabel


/* @param refParent is a reference to the corresponding node to parent.
 *                  It has to be NULL when calling this function.
 * @param refChild  is a reference to the corresponding node to child.
 *                  It has to be NULL when calling this function.
 */
static void 
findCorrespondingNodesInNewTree(Expr_node_ptr old_tree, Expr_node_ptr new_tree,
				Expr_node_ptr parent, Expr_node_ptr child, 
				Expr_node_ptr* refParent, 
				Expr_node_ptr* refChild)
{
  gint i;

  DBUG_ENTER("findCorrespondingNodesInNewTree");

  if ((old_tree != NULL) && (new_tree != NULL)) {
    if ((*refParent == NULL) && (parent == old_tree)) {
      *refParent = new_tree;
    } //if
    if ((*refChild == NULL) && (child == old_tree)) {
      *refChild = new_tree;
    } //if

    i = 0;
    while ( ( ( (*refParent == NULL) && (parent != NULL) ) || 
	      (*refChild == NULL) 
	    )  && 
	    (i < EXPR_NODE_MAX_CHILDREN) 
	  ) {
      findCorrespondingNodesInNewTree(old_tree->child[i], new_tree->child[i], 
				      parent, child, refParent, refChild);
      i++;
    } //while
  } //if

  DBUG_VOID_RETURN;
} //findCorrespondingNodesInNewTree


/* @return FALSE if nothing was evaluated, TRUE otherwise and
 *         in case of an RunTimeError
 */
gboolean 
gtk_eval(Drawable_expr_ptr pDrawExpr, Expr_node_ptr parent_of_esn, 
	 Expr_node_ptr eval_start_node,
	 EvalMode mode, gint nrMaxEvaluations, 
	 gboolean simplify_only, gboolean eval_if)
{
  Expr_node_ptr new_undo_expr, act_expr, 
                new_parent_of_esn, new_eval_start_node;
  gboolean dummy_created = FALSE, 
           res, noerr;
  gint child_index;

  DBUG_ENTER("gtk_eval");

  //prepare
  pDrawExpr->drawingAllowed = FALSE;
  fpUpdateStatistics = &update_statuslabel;
  new_undo_expr = pDrawExpr->act_expr;
  //copy the expr without the misc-structure
  act_expr      = duplicateSubtree(pDrawExpr->act_expr);
  pDrawExpr->act_expr  = NULL;

  //parent_of_esn and eval_start_node point into the old tree
  //but the new tree has to be evaluated!
  new_parent_of_esn   = NULL; //important for following functioncall
  new_eval_start_node = NULL; //important for following functioncall
  findCorrespondingNodesInNewTree(new_undo_expr, act_expr, 
				  parent_of_esn, eval_start_node,
				  &new_parent_of_esn, &new_eval_start_node);
  if (new_parent_of_esn == NULL) {
    new_parent_of_esn = create_dummy(); //create a dummy-node
    new_parent_of_esn->child[0] = new_eval_start_node;
    dummy_created = TRUE;
  } //if

  //to find the eval_start_node after evaluation later on
  eval_start_node_after_evaluation = NULL;
  child_index = getIndexOfChild(new_parent_of_esn, new_eval_start_node);
  DBUG_ASSERT(child_index != -1);

  //evaluate
  eval_stats.expansiondepth             = 0;
  eval_stats.max_reached_expansiondepth = 0;
  eval_stats.nr_expansions              = 0;
  eval_stats.nr_evaluations             = 0;

  if (eval_if) {
    noerr = evaluate(new_eval_start_node, COND_EXPR_NODE(new_eval_start_node),
		     mode, nrMaxEvaluations, simplify_only);
    noerr = noerr && evaluate(new_parent_of_esn, new_eval_start_node, 
			      mode, 1, simplify_only);
  } else {
    noerr = evaluate(new_parent_of_esn, new_eval_start_node, 
		     mode, nrMaxEvaluations, simplify_only);
  }
  new_eval_start_node = NULL; //it might have changed and freed
  if (dummy_created == TRUE) {
    act_expr = new_parent_of_esn->child[0];
    new_parent_of_esn->child[0] = NULL; //avoid freeing the child
    free_expr_node(new_parent_of_esn); //free the dummy
    new_parent_of_esn = NULL;
  } //if

  //was anything evaluated?
  res = eval_stats.nr_evaluations + eval_stats.nr_expansions;
  if ((res > 0) || (noerr == FALSE)) {
    //add to undo_list, set pDrawExpr->act_expr, free redo_list
    pDrawExpr->undo_list = g_list_prepend(pDrawExpr->undo_list, new_undo_expr);
    pDrawExpr->act_expr = act_expr;
    free_undoredo_list(pDrawExpr->redo_list);
    pDrawExpr->redo_list = NULL;
    if (new_parent_of_esn != NULL ) {
      eval_start_node_after_evaluation = new_parent_of_esn->child[child_index];
    } else {
      //new_parent_of_esn was a dummy which is allready freed.
      //Its only child was the act_expr.
      eval_start_node_after_evaluation = act_expr;
    }
  } else {
    //restore old status
    free_expr_node_inclusiv_misc(act_expr);
    pDrawExpr->act_expr = new_undo_expr;
    eval_start_node_after_evaluation = eval_start_node;
  } //if anything evaluated

  //update UndoRedo-window
  UndoRedo_ProgressBar_und_MaxLabel_aktualisieren(pDrawExpr);
  update_statuslabel();

  if (noerr ==  FALSE) {
    res = TRUE;
    Warnungs_Fenster_oeffnen(getErrorDescription_nr_buttons(), 
			     getErrorDescription_buttonlabel1(),
			     getErrorDescription_line0(), 
			     getErrorDescription_line1(), 
			     getErrorDescription_line2(), 
			     getErrorDescription_line3(),
			     getErrorDescription_line4(), 
			     getErrorDescription_line5(), NULL);
  } //if
  pDrawExpr->drawingAllowed = TRUE;

  DBUG_RETURN(res);
} //gtk_eval
