/* Dateiauswahl Fenster */
#include <errno.h>
#include "kiel_global.h"

#include "kiel_gtk_wf.h"
#include "kiel_gtk_df.h"
#include "kiel_gtk_df_priv.h"
#include "kiel_gtk_fenster.h"
#include "kiel_gtk.h"

#include "kiel_fkt.h"

#include "kiel_ressource.h"
#include "kiel_gtk_bmf.h"
#include "../Misc/error_interface.h"
#include "../Syntaxtree/syntaxtree.h"
#include "../Syntaxtree/st_io.h"
#include "drawable_expr.h"


static void 
GTK_Ausdruck(Drawable_expr_ptr pDrawExpr)
{ 
  GtkWidget *list_item;
  gchar *list_item_label;
  UEBERSICHTSFENSTER_TYP *adkuebersfensterP;

  kiel_debug_in(10,"GTK_Ausdruck");

  adkuebersfensterP = hole_adresse_von_adkuebersichtsfenster();

  /* RV-bersicht-Fenster aktualisieren */
  /* rv_liste neu fllen */
  list_item_label = exprNode2String(pDrawExpr->initial_expr);
  list_item = gtk_list_item_new_with_label(list_item_label);
  mtfree(list_item_label);
  gtk_container_add(GTK_CONTAINER((*adkuebersfensterP)->liste), list_item);
  gtk_widget_show(list_item);	
  gtk_object_set_user_data(GTK_OBJECT(list_item), pDrawExpr);

  kiel_debug_out(10,"GTK_Ausdruck");
}


void 
GTK_Ausdruecke(void)
{
  GList **MLAdksP;
  UEBERSICHTSFENSTER_TYP *adkuebersfensterP;

  kiel_debug_in(10,"GTK_Ausdruecke");
 
  MLAdksP = hole_adresse_von_MLAusdruecke();
  adkuebersfensterP = hole_adresse_von_adkuebersichtsfenster();

  /* Ausdruck-Liste leeren */
  gtk_list_remove_items(GTK_LIST((*adkuebersfensterP)->liste),
                        GTK_LIST((*adkuebersfensterP)->liste)->children);
  /* Ausdrcke einfgen */
  g_list_foreach(*MLAdksP, (GFunc)GTK_Ausdruck, NULL);  

  kiel_debug_out(10,"GTK_Ausdruecke");
}


static void 
GTK_Rechenvorschrift(Fun_node_ptr pFun, gboolean insertDatatypeline)
{
  GtkWidget *list_item;
  gchar *list_item_label;
  UEBERSICHTSFENSTER_TYP *rvuersfensterP;

  kiel_debug_in(10,"GTK_Rechenvorschrift");

  rvuersfensterP = hole_adresse_von_rvuebersichtsfenster();

  /* RV-bersicht-Fenster aktualisieren */
  /* rv_liste neu fllen */
  if (insertDatatypeline) {
    list_item_label = funNode2DatatypeStr(pFun);;
    list_item = gtk_list_item_new_with_label(list_item_label);
    mtfree(list_item_label);
    gtk_container_add(GTK_CONTAINER((*rvuersfensterP)->liste), list_item);
    gtk_widget_show(list_item);	
    gtk_object_set_user_data(GTK_OBJECT(list_item), NULL);
  } //if
  list_item_label = funNode2Signature(pFun);
  list_item = gtk_list_item_new_with_label(list_item_label);
  mtfree(list_item_label);
  gtk_container_add(GTK_CONTAINER((*rvuersfensterP)->liste), list_item);
  gtk_widget_show(list_item);	
  gtk_object_set_user_data(GTK_OBJECT(list_item), pFun);

  kiel_debug_out(10,"GTK_Rechenvorschrift");
}


static void 
GTK_Rechenvorschriften(void)
{
  //GList **MLRVsP;
  UEBERSICHTSFENSTER_TYP *rvuersfensterP;
  GList* tmpST;
  Fun_node_ptr pFun;
  gint last_resulttype = -1;

  kiel_debug_in(10,"GTK_Rechenvorschriften");

  //MLRVsP = hole_adresse_von_MLRechenvorschriften();
  rvuersfensterP = hole_adresse_von_rvuebersichtsfenster();

  /* Rumpfansicht-Liste leeren */
  gtk_list_remove_items(GTK_LIST((*rvuersfensterP)->liste),
                        GTK_LIST((*rvuersfensterP)->liste)->children);
  /* RVs einfgen */
  //g_list_foreach(syntax_tree, (GFunc)GTK_Rechenvorschrift,NULL);
  for (tmpST = syntax_tree;  tmpST != NULL;  tmpST = g_list_next(tmpST)) {
    pFun = tmpST->data;
    if (pFun->nodetype == e_constructor) {
      GTK_Rechenvorschrift(pFun, last_resulttype != pFun->resulttype);
      last_resulttype = pFun->resulttype;
    } else {
      GTK_Rechenvorschrift(pFun, FALSE);
      last_resulttype = -1;
    } //if-else
  } //for

  kiel_debug_out(10,"GTK_Rechenvorschriften");
}


/* @return NULL in case of an error or the contents of an ascii-file
 *         as a string of new allocated memory
 */
static gchar* 
loadFileIntoString(gchar* filename)
{
  const gint bufsize = 8;
  gint actsize, count;
  gchar c, *res;
  FILE* fp;

  fp = fopen(filename, "r");
  if (fp != NULL) {
    count   = 0;
    actsize = bufsize;
    res = mtmalloc(sizeof(gchar) * actsize);
    while ((c = getc(fp)) != EOF) {
      count++;
      if (count >= actsize) {
	actsize += bufsize;
	res = mtrealloc(res, sizeof(gchar) * actsize);
      }
      res[count-1] = c;
    } //while
    res[count] = '\0'; //terminate string
  } else {
    res = NULL;
    Warnungs_Fenster_oeffnen(1, BUTTON_WARNUNG_OK_TEXT,
			     "Can't open file:", filename,
			     strerror(errno), NULL);
  }

  return res;
} //loadFileIntoString


void Ausdruck_Laden(char *datname)
{
  gchar *exprStr;
  GList **MLAdksP;
  GList **MLRVsP;
  Expr_node_ptr pExpr;
  Drawable_expr_ptr pDrawExpr;

  kiel_debug_in(10,"Ausdruck_FileSel_OK");

    MLAdksP = hole_adresse_von_MLAusdruecke();
    MLRVsP  = hole_adresse_von_MLRechenvorschriften();

      exprStr = loadFileIntoString(datname);
      
printf("%s\n", exprStr);
      if (exprStr != NULL) {
	pExpr = readExprFromString(exprStr);
	mtfree(exprStr);
	if (pExpr != NULL) {
	  pDrawExpr = create_drawable_expr(pExpr);
	  pExpr = NULL;
	  *MLAdksP = g_list_append(*MLAdksP, pDrawExpr);
	  GTK_Ausdruecke();

	  /* Ausdruck-bersichtsfenster anzeigen */
	  if (*MLAdksP) {
	    trotz_warnung_auf_true();
	    ADK_Uebersichtsfenster_Toggle_Button_an();
	    trotz_warnung_auf_false();
	  } //if (*MLAdksP)
	} else {
	  Warnungs_Fenster_oeffnen(getErrorDescription_nr_buttons(), 
				   getErrorDescription_buttonlabel1(),
				   getErrorDescription_line0(), 
				   getErrorDescription_line1(), 
				   getErrorDescription_line2(), 
				   getErrorDescription_line3(),
				   getErrorDescription_line4(), 
				   getErrorDescription_line5(), NULL);
	} //if (pExpr != NULL)


      } //if (exprStr != NULL)

  kiel_debug_out(10,"Ausdruck_FileSel_OK");
}


void Ausdruck_FileSel_OK(GtkWidget *w,GtkFileSelection *fs)
{
  gchar *datname, *exprStr;
  gboolean gueltige_auswahl;
  GList **MLAdksP;
  GList **MLRVsP;
  Expr_node_ptr pExpr;
  Drawable_expr_ptr pDrawExpr;

  kiel_debug_in(10,"Ausdruck_FileSel_OK");

  if (warnungs_fenster_geoeffnet()) beep();
  else {
    MLAdksP = hole_adresse_von_MLAusdruecke();
    MLRVsP  = hole_adresse_von_MLRechenvorschriften();

    datname = mtstrdup(gtk_file_selection_get_filename(fs));
    if      (strlen(datname) < 4) 
      gueltige_auswahl = FALSE;
    else if (strcmp(&datname[strlen(datname) - 4], ".exp") != 0) 
      gueltige_auswahl=FALSE;
    else 
      gueltige_auswahl=TRUE;

    if (gueltige_auswahl) {
      gtk_widget_destroy(GTK_WIDGET(fs));

      exprStr = loadFileIntoString(datname);
printf("%s\n", exprStr);
      if (exprStr != NULL) {
	pExpr = readExprFromString(exprStr);
	mtfree(exprStr);
	if (pExpr != NULL) {
	  pDrawExpr = create_drawable_expr(pExpr);
	  pExpr = NULL;
	  *MLAdksP = g_list_append(*MLAdksP, pDrawExpr);
	  GTK_Ausdruecke();

	  /* Ausdruck-bersichtsfenster anzeigen */
	  if (*MLAdksP) {
	    trotz_warnung_auf_true();
	    ADK_Uebersichtsfenster_Toggle_Button_an();
	    trotz_warnung_auf_false();
	  } //if (*MLAdksP)
	} else {
	  Warnungs_Fenster_oeffnen(getErrorDescription_nr_buttons(), 
				   getErrorDescription_buttonlabel1(),
				   getErrorDescription_line0(), 
				   getErrorDescription_line1(), 
				   getErrorDescription_line2(), 
				   getErrorDescription_line3(),
				   getErrorDescription_line4(), 
				   getErrorDescription_line5(), NULL);
	} //if (pExpr != NULL)


      } //if (exprStr != NULL)
    } else {
      gchar *z;

      z=g_strdup_printf(TEXT_ADK_DATNAME1,datname);
      Warnungs_Fenster_oeffnen(1,BUTTON_WARNUNG_OK_TEXT,
                                 TEXT_ADK_DATNAME0,z,
                                 TEXT_ADK_DATNAME2,TEXT_ADK_DATNAME3,NULL);

      g_free(z);
    }
    mtfree(datname);
  }

  kiel_debug_out(10,"Ausdruck_FileSel_OK");
}


void Programm_FileSel_OK(GtkWidget *w,GtkFileSelection *fs)
{
  gchar *datname;
  gboolean gueltige_auswahl;
  gint err;
  GList **MLRVsP;
  GList **MLAdksP;
  BAUMFENSTER_TYP *bmfensterP;

  kiel_debug_in(10,"Programm_FileSel_OK");
  
  if (warnungs_fenster_geoeffnet()) beep();
  else {
    MLRVsP  = hole_adresse_von_MLRechenvorschriften();
    MLAdksP = hole_adresse_von_MLAusdruecke();

    datname = g_strdup(gtk_file_selection_get_filename(fs));
    DBUG_PRINT("gtk_df", ("%s", datname));
    if (strlen(datname) < 4) gueltige_auswahl = FALSE;
    else if (strcmp(&datname[strlen(datname)-4],".prg")!=0) 
           gueltige_auswahl=FALSE;
    else gueltige_auswahl=TRUE;

    if (gueltige_auswahl) {
      gtk_widget_destroy(GTK_WIDGET(fs));

      err = readPrgFromFile(datname);
      if (err == 0) {
	GTK_Rechenvorschriften();
	loesche_MLAusdruecke();
	GTK_Ausdruecke();

	//lsche Baumfenster
	bmfensterP = hole_adresse_von_baumfenster();
	if ( (*bmfensterP) && ((*bmfensterP)->pix_draw->pixm) ) {
	  baumfenster_configure_event((*bmfensterP)->pix_draw->draw_a, NULL);
	}
        trotz_warnung_auf_true();
        RV_Uebersichtsfenster_Toggle_Button_an();
        trotz_warnung_auf_false();
      } else {
	Warnungs_Fenster_oeffnen(getErrorDescription_nr_buttons(), 
				 getErrorDescription_buttonlabel1(),
				 getErrorDescription_line0(), 
				 getErrorDescription_line1(), 
				 getErrorDescription_line2(), 
				 getErrorDescription_line3(),
				 getErrorDescription_line4(), 
				 getErrorDescription_line5(), NULL);
      }

    } else {
      gchar *z;

      z=g_strdup_printf(TEXT_PRG_DATNAME1,datname);
      Warnungs_Fenster_oeffnen(1,BUTTON_WARNUNG_OK_TEXT,
                                 TEXT_PRG_DATNAME0,z,
                                 TEXT_PRG_DATNAME2,TEXT_PRG_DATNAME3,NULL);

      g_free(z);
    }
    g_free(datname);
  }

  kiel_debug_out(10,"Programm_FileSel_OK");
}

void Programm_Laden(gchar *datname)
{
  gint err;
  GList **MLRVsP;
  GList **MLAdksP;
  BAUMFENSTER_TYP *bmfensterP;

  kiel_debug_in(10,"Programm_FileSel_OK");
  
    MLRVsP  = hole_adresse_von_MLRechenvorschriften();
    MLAdksP = hole_adresse_von_MLAusdruecke();

    DBUG_PRINT("gtk_df", ("%s", datname));

      err = readPrgFromFile(datname);
      if (err == 0) {
	GTK_Rechenvorschriften();
	loesche_MLAusdruecke();
	GTK_Ausdruecke();

	//lsche Baumfenster
	bmfensterP = hole_adresse_von_baumfenster();
	if ( (*bmfensterP) && ((*bmfensterP)->pix_draw->pixm) ) {
	  baumfenster_configure_event((*bmfensterP)->pix_draw->draw_a, NULL);
	}
        trotz_warnung_auf_true();
        RV_Uebersichtsfenster_Toggle_Button_an();
        trotz_warnung_auf_false();
      } else {
	Warnungs_Fenster_oeffnen(getErrorDescription_nr_buttons(), 
				 getErrorDescription_buttonlabel1(),
				 getErrorDescription_line0(), 
				 getErrorDescription_line1(), 
				 getErrorDescription_line2(), 
				 getErrorDescription_line3(),
				 getErrorDescription_line4(), 
				 getErrorDescription_line5(), NULL);
      }



  kiel_debug_out(10,"Programm_FileSel_OK");
}

void widget_zerstoeren(GtkWidget *widget,gpointer *data)
{
  kiel_debug_in(10,"widget_zerstoeren");
 
  if (warnungs_fenster_geoeffnet()) beep();
  else {
    gtk_widget_destroy((GtkWidget *)data);
  }

  kiel_debug_out(10,"widget_zerstoeren");
}


void file_selection_destroy(GtkWidget *widget,gpointer *data)
{
  kiel_debug_in(10,"file_selection_destroy");
 
  *data=NULL;

  kiel_debug_out(10,"file_selection_destroy");
}


void Ausdruck_Dateiauswahl_oeffnen(void)
{
  FILE *dat;

  kiel_debug_in(10,"Ausdruck_Dateiauswahl_oeffnen");

  if (!adk_file_selection_widget) {
    /* File Selection Widget erzeugen */
    adk_file_selection_widget=
      gtk_file_selection_new(WINDOW_AUSDRUCK_DATEIAUSWAHL_TEXT);
    gtk_widget_set_uposition(adk_file_selection_widget,
                             POSITION_ADKFILESELECTION_X,
                             POSITION_ADKFILESELECTION_Y);
    gtk_container_border_width(GTK_CONTAINER(adk_file_selection_widget),5);

    gtk_signal_connect(GTK_OBJECT(adk_file_selection_widget),"destroy",
                       GTK_SIGNAL_FUNC(file_selection_destroy),
                       &adk_file_selection_widget);

    gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(adk_file_selection_widget)
                                                     ->ok_button),
                       "clicked",GTK_SIGNAL_FUNC(Ausdruck_FileSel_OK),
                       adk_file_selection_widget);

    gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(
                                    adk_file_selection_widget)->cancel_button),
                       "clicked",GTK_SIGNAL_FUNC(widget_zerstoeren),
                       adk_file_selection_widget);
    dat = fopen(WINDOW_AUSDRUCK_DATEIAUSWAHL_DIRECTORY, "r");
    if (dat != NULL) {
      gtk_file_selection_set_filename(
	GTK_FILE_SELECTION(adk_file_selection_widget),
        WINDOW_AUSDRUCK_DATEIAUSWAHL_DIRECTORY);
      fclose(dat);
    } else {
      gtk_file_selection_set_filename(
        GTK_FILE_SELECTION(adk_file_selection_widget),
        ".");
    }

    gtk_file_selection_hide_fileop_buttons(GTK_FILE_SELECTION(
                                           adk_file_selection_widget));

    gtk_widget_show(adk_file_selection_widget);
  }
  else {
    Warnungs_Fenster_oeffnen(1,BUTTON_WARNUNG_OK_TEXT,
                               TEXT_ADK_FENSTER0,TEXT_ADK_FENSTER1,NULL);
  }

  kiel_debug_out(10,"Ausdruck_Dateiauswahl_oeffnen");
}

 
void Programm_Dateiauswahl_oeffnen(void)
{
  FILE *dat;

  kiel_debug_in(10,"Programm_Dateiauswahl_oeffnen");

  if (!prg_file_selection_widget) {
    /* File Selection Widget erzeugen */

    prg_file_selection_widget=
      gtk_file_selection_new(WINDOW_PROGRAMM_DATEIAUSWAHL_TEXT);

    gtk_widget_set_uposition(prg_file_selection_widget,
                             POSITION_PRGFILESELECTION_X,
                             POSITION_PRGFILESELECTION_Y); 
    gtk_container_border_width(GTK_CONTAINER(prg_file_selection_widget),5);

    gtk_signal_connect(GTK_OBJECT(prg_file_selection_widget),"destroy",
                       GTK_SIGNAL_FUNC(file_selection_destroy),
                       &prg_file_selection_widget);

    gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(prg_file_selection_widget)
                                                     ->ok_button),
                       "clicked",GTK_SIGNAL_FUNC(Programm_FileSel_OK),
                       prg_file_selection_widget);

    gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(
                                    prg_file_selection_widget)->cancel_button),
                       "clicked",GTK_SIGNAL_FUNC(widget_zerstoeren),
                       prg_file_selection_widget);
    dat = fopen(WINDOW_PROGRAMM_DATEIAUSWAHL_DIRECTORY, "r");
    if (dat != NULL) {
      gtk_file_selection_set_filename(
        GTK_FILE_SELECTION(prg_file_selection_widget),
        WINDOW_PROGRAMM_DATEIAUSWAHL_DIRECTORY);
      fclose(dat);
    } else {
      gtk_file_selection_set_filename(
        GTK_FILE_SELECTION(prg_file_selection_widget),
        ".");
    }

    gtk_file_selection_hide_fileop_buttons(GTK_FILE_SELECTION(
                                           prg_file_selection_widget));

    gtk_widget_show(prg_file_selection_widget);
  } else {
    Warnungs_Fenster_oeffnen(1,BUTTON_WARNUNG_OK_TEXT,
                               TEXT_PRG_FENSTER0,TEXT_PRG_FENSTER1,NULL);
  }

  kiel_debug_out(10,"Programm_Dateiauswahl_oeffnen");
}
