/*
*  C Implementation: window_options
*
* Description: 
*
*
* Author: pm <pm@ariva.de>, (C) 2006
*
* Copyright: See COPYING file that comes with this distribution
*
*/
#include <stdio.h>
#include "window_options.h"
#include "../Misc/mem_management.h"
#include "kiel_global.h"
#include "kiel_gtk_fenster.h"
#include "kiel_gtk_bf.h"
#include "kiel_gtk_df.h"
#include "kiel_gtk_uef.h"
#include "kiel_gtk_wf.h"
#include "kiel_gtk.h"

WIN_OPTIONS win_options_new() {
  WIN_OPTIONS n = mtmalloc(sizeof(WIN_OPTIONS_STRUCT));
  return n;
}

WIN_OPTIONS win_options_new_from_fd(FILE *fd) {
  WIN_OPTIONS n = win_options_new();
  fread(n,sizeof(WIN_OPTIONS_STRUCT),1,fd);
  return n;
}

void win_options_save_to_fd(WIN_OPTIONS opt, FILE *fd) {
  fwrite(opt,sizeof(WIN_OPTIONS_STRUCT),1,fd);
}



OPTIONS options_new() {
  OPTIONS n = mtmalloc(sizeof(OPTIONS_STRUCT));
  return n;
}

OPTIONS options_new_from_fd(FILE *fd) {
  OPTIONS n = options_new();
  n->bf = win_options_new_from_fd(fd);
  n->urf = win_options_new_from_fd(fd);
  n->aef = win_options_new_from_fd(fd);
  n->awf = win_options_new_from_fd(fd);
  n->bmf = win_options_new_from_fd(fd);
  n->ueber_rv = win_options_new_from_fd(fd);
  n->ueber_ex = win_options_new_from_fd(fd);
  return n;
}

void options_save_to_fd(OPTIONS opt, FILE *fd) {

  win_options_save_to_fd(opt->bf,fd);
  win_options_save_to_fd(opt->urf,fd);
  win_options_save_to_fd(opt->aef,fd);
  win_options_save_to_fd(opt->awf,fd);
  win_options_save_to_fd(opt->bmf,fd);
  win_options_save_to_fd(opt->ueber_rv,fd);
  win_options_save_to_fd(opt->ueber_ex,fd);
  
}



void set_window_options(WIN_OPTIONS wopt, GtkWidget *window) {
  gtk_widget_set_uposition(window,wopt->pos_x,wopt->pos_y);
  g_print("\n\nwill resize\n");
  g_print("resize to %i x %i\n", wopt->width,wopt->height);
  if ( wopt->visible == 0 )  {
    gtk_widget_show(window);
    gdk_window_resize(GTK_WIDGET(window)->window,wopt->width,wopt->height);
    gtk_widget_hide(window);
  } else  {
    gtk_widget_show(window);
    gdk_window_resize(GTK_WIDGET(window)->window,wopt->width,wopt->height);
  }
  g_print("resize to %i x %i\n", wopt->width,wopt->height);
  
        
}

void get_window_options(WIN_OPTIONS wopt, GtkWidget *window) {
  gint x, y, w, h;
  gdk_window_get_origin (GTK_WIDGET(window)->window, &x, &y);
  gdk_window_get_size   (GTK_WIDGET(window)->window, &w, &h);
  wopt->pos_x = x;
  wopt->pos_y = y;
  wopt->width = w;
  wopt->height = h;
  wopt->visible =   GTK_WIDGET_VISIBLE(window) ? 1 : 0;
}

char *config_filename() {
  char *home = mtstrdup(getenv("HOME"));
  char *more = mtstrdup("/.kielrc");
  return mtstrconcat(2,home,more);
}

void options_load() {
  char *filename = config_filename();

  FILE *cfile = fopen(filename,"r");
  mtfree(filename);
  if ( cfile == NULL ) {
    g_print("can't open config.\n");
  } else {
    OPTIONS opt = options_new_from_fd(cfile);

    set_window_options(opt->bf, (*(hole_adresse_von_befehlsfenster()))->fenster);
    set_window_options(opt->urf, (*(hole_adresse_von_undoredofenster()))->fenster);
    set_window_options(opt->aef, (*(hole_adresse_von_ausdruckeingabefenster() ))->fenster);
    set_window_options(opt->awf, (*( hole_adresse_von_auswertungsfenster()))->fenster);
    set_window_options(opt->bmf, (*(hole_adresse_von_baumfenster() ))->fenster);
    set_window_options(opt->ueber_rv, (*(hole_adresse_von_rvuebersichtsfenster()))->fenster);
    set_window_options(opt->ueber_ex, (*(hole_adresse_von_adkuebersichtsfenster()))->fenster);
        
    g_print("options read\n");
    g_print("int %i\n",opt->bf->pos_x);
    fclose(cfile);
  }
}


void options_save() {
  char *filename = config_filename();
  FILE *cfile = fopen(filename,"w+");
  mtfree(filename);
  if ( cfile == NULL ) {
    g_print("can't open config for writing.\n");
  } else {
    OPTIONS opt = options_new();
    
    opt->bf = win_options_new();
    opt->urf= win_options_new();
    opt->aef= win_options_new();
    opt->awf= win_options_new();
    opt->bmf= win_options_new();
    opt->ueber_rv= win_options_new();
    opt->ueber_ex= win_options_new();
    
    get_window_options(opt->bf, (*(hole_adresse_von_befehlsfenster()))->fenster);
    get_window_options(opt->urf, (*(hole_adresse_von_undoredofenster()))->fenster);
    get_window_options(opt->aef, (*(hole_adresse_von_ausdruckeingabefenster() ))->fenster);
    get_window_options(opt->awf, (*( hole_adresse_von_auswertungsfenster()))->fenster);
    get_window_options(opt->bmf, (*(hole_adresse_von_baumfenster() ))->fenster);
    get_window_options(opt->ueber_rv, (*(hole_adresse_von_rvuebersichtsfenster()))->fenster);
    get_window_options(opt->ueber_ex, (*(hole_adresse_von_adkuebersichtsfenster()))->fenster);

    options_save_to_fd(opt,cfile);
    fclose(cfile);
  }
}

