#include <stdio.h>
#include <string.h>
#include <gtk/gtk.h>
#include "../Misc/my_dbug.h"
#include "../Misc/mem_management.h" //for dbugging only
#include "../Misc/error_interface.h"
#include "../Typechecker/typechecker.h"
#include "syntaxtree.h"


extern FILE* prgin;

extern void prg_Initialize(void);
extern int  prgparse();
extern void prg_Finalize(void);
extern void expr_Initialize(void);
extern int  exprparse();
extern void expr_Finalize(void);
extern gchar* expr_input;
extern gchar* expr_inputptr;
extern gchar* expr_inputlim;



/* @return 1 in case of an error and 0 otherwise.
 */
gint 
readPrgFromFile(gchar* filename)
{
  gint res = 0;
  GList* old_syntax_tree;
  GList* old_symbol_table;

  DBUG_ENTER("readPrgFromFile");

  //prgdebug = 0; // 0 == no debugoutput; 0 != do debugoutput
  prgin = fopen(filename, "r");
  DBUG_PRINT("gtk_df", ("    %s", filename));
  if (prgin == NULL) { DBUG_RETURN(1); }
  old_syntax_tree  = syntax_tree;
  old_symbol_table = symbol_table;
  syntax_tree  = NULL;
  symbol_table = NULL;
  prg_Initialize(); mtshowtotals("after prg_initialize");
  { 
    DBUG_ENTER("prgparse");
    res = prgparse(); mtshowtotals("after prgparse");
    DBUG_LEAVE;
  }
  prg_Finalize();   mtshowtotals("after prg_finalize");
  fclose(prgin);

  if (res == 0) {
    res = typecheck_prg(syntax_tree);
    if (res == 0) {
      free_syntaxtree( old_syntax_tree);
      free_symboltable(old_symbol_table);
    } else {
      free_syntaxtree( syntax_tree);
      free_symboltable(symbol_table);
      syntax_tree  = old_syntax_tree;
      symbol_table = old_symbol_table;
      printf_ErrorDescription();
    }
  } else {
    printf_ErrorDescription();
    /**** dbug_only ****/
    //nav_syntaxtree(syntax_tree);

    free_syntaxtree( syntax_tree);
    free_symboltable(symbol_table);
    syntax_tree  = old_syntax_tree;
    symbol_table = old_symbol_table;
  }

  DBUG_RETURN(res);
} //readPrgFromFile


/* @return NULL in case of an error and a syntaxtree for the expression 
 *         otherwise.
 */
Expr_node_ptr
readExprFromString(gchar* inputstr)
{
  gint err = 0;
  Expr_node_ptr res = NULL; 

  DBUG_ENTER("readExprFromString");

  //exprdebug = 0; // 0 == no debugoutput; 0 != do debugoutput
  expr_input    = inputstr;
  expr_inputptr = expr_input;
  expr_inputlim = expr_input + strlen(expr_input);

  expr_Initialize(); mtshowtotals("after expr_initialize");
  err = exprparse(); mtshowtotals("after exprparse");
  expr_Finalize();   mtshowtotals("after expr_finalize");

  if (err == 0) {
	printf("length %i\n", g_list_length(tree_parts));
	printf_treeparts(tree_parts);
		 
    DBUG_ASSERT(g_list_length(tree_parts) == 1);
    res = tree_parts->data;
    tree_parts = myGlistRemove(tree_parts, res);
    DBUG_ASSERT(tree_parts == NULL);

    //nav_expr_node(res);

    err = typecheck_expr(res);
    if (err == 1) {
      printf_ErrorDescription();
      free_expr_node(res);
      res = NULL;
    }
  } else {
    printf_ErrorDescription();
    tree_parts = free_treeparts(tree_parts);
    res = NULL;
    {
      /**** dbug_only ****/
      //nav_syntaxtree(syntax_tree);
    }
  }

  DBUG_RETURN(res);
} //readExprFromString
