#include <stdio.h>
#include <string.h>
#include "../Misc/my_dbug.h"
#include "../Misc/utilities.h"
#include "../Misc/mem_management.h"
#include "symboltable.h"



GList* symbol_table = NULL;
GList* undefined_symbols = NULL;
GList* binding_table = NULL;



Symboltable_entry_ptr
seekSymbolId(GList* table, gchar* id)
{
  int found = 0;
  Symboltable_entry_ptr pSte = NULL;

  DBUG_ENTER("seekSymbolId");
  DBUG_ASSERT(id != NULL);

  while ((table != NULL) && (table->data != NULL) && !found){
    pSte = table->data;
    if (0 == strcmp(SYMBOL_ID(pSte), id)) {
      found = 1;
    } else {
      table = g_list_next(table);
    }
  } //while

  if (!found) {
    pSte = NULL;
    DBUG_PRINT("seek", ("id '%s' not found", id));
  }

  DBUG_RETURN(pSte);
}//seekSymbolId


Symboltable_entry_ptr
seekSymbolTypeindex(GList* table, gint type_index)
{
  int found = 0;
  Symboltable_entry_ptr pSte = NULL;

  DBUG_ENTER("seekSymbolTypeindex");

  while ((table != NULL) && (table->data != NULL) && !found){
    pSte = table->data;
    if ( ( (SYMBOL_SYMBOL_TYPE(pSte) == e_st_basictype) ||
	    (SYMBOL_SYMBOL_TYPE(pSte) == e_st_product) ||
           (SYMBOL_SYMBOL_TYPE(pSte) == e_st_datatype)
         ) && (SYMBOL_TYPE_INDEX(pSte) == type_index)
       ) {
      found = 1;
    } else {
      table = g_list_next(table);
    }
  } //while

  if (!found) {
    pSte = NULL;
    DBUG_PRINT("seek", ("type_index '%i' not found", type_index));
  }

  DBUG_RETURN(pSte);
} //seekSymbolTypeindex


Symboltable_entry_ptr
seekSymbolTypeOfListitems(GList* table, gint type_of_listitems)
{
  int found = 0;
  Symboltable_entry_ptr pSte = NULL;

  DBUG_ENTER("seekSymbolTypeOfListitems");

  while ((table != NULL) && (table->data != NULL) && !found){
    pSte = table->data;
    if ( (SYMBOL_SYMBOL_TYPE(pSte) == e_st_basictype) 
         && (SYMBOL_TYPE_OF_LISTITEMS(pSte) == type_of_listitems)
       ) {
      found = 1;
    } else {
      table = g_list_next(table);
    }
  } //while

  if (!found) {
    pSte = NULL;
    DBUG_PRINT("seek", ("type_of_listitems '%i' not found", 
                        type_of_listitems));
  }

  DBUG_RETURN(pSte);
} //seekSymbolTypeOfListitems


GList* 
addSymbol(GList* table, gchar* id, Symbol_type symbol_type, gint nr_params,
          gint type_index, gint type_of_listitems, void* definition_point)
{
  gint nesting = 0;
  GList* new_table;
  Symboltable_entry_ptr pEntry, tmp_pEntry;

  DBUG_ENTER("addSymbol");
  DBUG_ASSERT(id != NULL);

  pEntry = mtmalloc(sizeof(Symboltable_entry_struct));
  pEntry->id               = mtstrdup(id);
  pEntry->symbol_type      = symbol_type;
  pEntry->nr_params        = nr_params;
  pEntry->type_index       = type_index;
  pEntry->type_of_listitems= type_of_listitems;
  pEntry->definition_point = definition_point;
  //calc the listnesting
  tmp_pEntry = pEntry;
  while (tmp_pEntry->type_of_listitems != 0) {
    nesting++;
    tmp_pEntry = seekSymbolTypeindex(table, tmp_pEntry->type_of_listitems);
    DBUG_ASSERT(tmp_pEntry != NULL);
  } //while
  pEntry->listnesting = nesting;
  /*printf("listnesting for '%s' (%d) is %d\n", pEntry->id, pEntry->type_index,
    pEntry->listnesting);*/
  //add to table
  new_table = g_list_append(table, pEntry);

  DBUG_RETURN(new_table);
}//addSymbol


GList* 
free_symboltable(GList* table)
{
  GList* pElm;
  Symboltable_entry_ptr pSte;

  DBUG_ENTER("free_symboltable");

  pElm = table;
  while (pElm != NULL){
    pSte = pElm->data;
    if (pSte != NULL) {
      MTFREENULL(pSte->id);
      pSte->definition_point = NULL;
      MTFREENULL(pElm->data);
    }
    pElm = g_list_next(pElm);
  } //while
  g_list_free(table);

  DBUG_RETURN((GList*)NULL);
}//free_symboltable


void
printf_symboltable(GList* table)
{
  GList* tmp;
  Symboltable_entry_ptr pSte;

  DBUG_ENTER("printf_symboltable");

  for (tmp = table; tmp != NULL; tmp = g_list_next(tmp)) {
    pSte = tmp->data;
    printf("%s\tsymbol_type: %i\ttype_index: %i\n", 
           pSte->id, pSte->symbol_type, pSte->type_index);
  } //for

  DBUG_VOID_RETURN;
}//printf_symboltable

Product_part_ptr
createProduct_part(gint type) {
	Product_part_ptr pEntry;
	pEntry = mtmalloc(sizeof(Product_part_struct));	
	pEntry->type = type;
	pEntry->next =NULL;
	return pEntry;
}

void 
freeProduct_part(Product_part_ptr pEntry) {
	if ( pEntry->next != NULL ) {
		freeProduct_part(pEntry->next);
	}
	mtfree(pEntry);
}

void 
appendProduct_part(Product_part_ptr pEntry, gint type) {
	Product_part_ptr new;
	// go to last entry, we append at the end and don't care about performance
	while ( pEntry->next != NULL ) { 
		pEntry = pEntry->next ;
	}

	new = createProduct_part(type);
	pEntry->next = new;

}

Product_part_ptr
prependProduct_part(Product_part_ptr pEntry, gint type) {
	Product_part_ptr new = createProduct_part(type);
	new->next = pEntry;	
	return new;
}

