#ifndef _SYMBOLTABLE_H
#define _SYMBOLTABLE_H



#include <gtk/gtk.h>


typedef enum { e_st_unknown, e_st_basicfun, e_st_userfun, e_st_constructor,
               e_st_basictype, e_st_datatype, e_st_param, e_st_product
} Symbol_type;


typedef struct _Product_part_struct {
  gint               type;
  struct Product_part_ptr   *next;
} Product_part_struct, *Product_part_ptr;

typedef struct {
  gchar*       id;               //The id as string.
  Symbol_type  symbol_type;      //The kind of id.
  gint         nr_params;        //Number of parameters (in case of a function 
                                 //or a constuctor)
  gint         type_index;       //type_index (e.g. 1=unit, 2=bool)
  gint         type_of_listitems;//type_index is a list with items of the type 
                                 //stored here
                                 //0 means is not a list
  gint         listnesting;      //'int'=0, 'int list'=1, 'int list list'=2,...
  gpointer     definition_point; //pointer to a funnode within the syntaxtree
  Product_part_struct* product_parts;
} Symboltable_entry_struct, *Symboltable_entry_ptr;


/* 'symbol_table' store all symbols which are allready defined. But in systems 
 * of functions or datatypes symbols may be used, which will be defined later
 * within the system. They are stored in 'undefined_symbols' and when the 
 * system has been completly parsed, all symbols in 'undefined_symbols' will
 * be checked for beeing stored in 'symbol_table' now.
 * To avoid new functions for the 'undefined_symbols' the 
 * Symboltable_entry_struct is reused, but because other data is stored,
 * special acces-macros are used.
 * The 'param_table' stores parameternames to check if only these parameters
 * are used within the body of the function.
 */
extern GList* symbol_table;      //list of Symboltable_entry_ptr's
extern GList* undefined_symbols; //list of Symboltable_entry_ptr's
extern GList* binding_table;     //list of Symboltable_entry_ptr's

#define SYMBOL_ID(pSte)                (pSte->id)
#define SYMBOL_SYMBOL_TYPE(pSte)       (pSte->symbol_type)
#define SYMBOL_NR_PARAMS(pSte)         (pSte->nr_params)
#define SYMBOL_TYPE_INDEX(pSte)        (pSte->type_index)
#define SYMBOL_TYPE_OF_LISTITEMS(pSte) (pSte->type_of_listitems)
#define SYMBOL_LISTNESTING(pSte)       (pSte->listnesting)
#define SYMBOL_DEFINITION_POINT(pSte)  (pSte->definition_point)

#define SYMBOL_UNDEF_ID(pSte)          (pSte->id)
#define SYMBOL_UNDEF_TYPE_INDEX(pSte)  (pSte->type_index)
#define SYMBOL_UNDEF_LINENO(pSte)      (pSte->nr_params)
//do not use pSte->type_of_listitems or pSte->listnesting 
//for SYMBOL_UNDEF_COLNO or SYMBOL_UNDEF_LINENO!
#define SYMBOL_UNDEF_COLNO(pSte)       (pSte->definition_point)



extern Symboltable_entry_ptr
seekSymbolId(GList* table, gchar* id);

extern Symboltable_entry_ptr
seekSymbolTypeindex(GList* table, gint type_index);

extern Symboltable_entry_ptr
seekSymbolTypeOfListitems(GList* table, gint type_of_listitems);

extern GList* 
addSymbol(GList* table, gchar* id, Symbol_type symbol_type, gint nr_params,
          gint type_index, gint type_of_listitems, void* definition_point);

extern GList*
free_symboltable(GList* table);

extern void
printf_symboltable(GList* table);


Product_part_ptr
createProduct_part(gint type);

void 
freeProduct_part(Product_part_ptr pEntry);

void 
appendProduct_part(Product_part_ptr pEntry, gint type);

Product_part_ptr
prependProduct_part(Product_part_ptr pEntry, gint type);
#endif //_SYMBOLTABLE_H
